import 'package:flutter/cupertino.dart';
import 'package:nui_core/config/nui_global_config.dart';
import 'package:nui_core/nui_core.dart';

/**
 * Data class to hold the required information for each of the language supported
 */
class NUIMTLanguage{
  NUIMTLanguage({required this.code, this.assetFilename, this.dictionary, this.externalAssetFilenames});
  String code; // Self-defined language code (Unique for each language)
  String? assetFilename; // Asset file name to import the language dictionary
  List<String>? externalAssetFilenames; // [2023/10/26] New field to allow importing multiple asset file names to be imported from external packages
  Map<String, String>? dictionary; // Direct import of the language dictionary
}

String? mtl(String code){
  return NUIMTLingual.get()?.lang(code);
}

/**
 * Singleton instance of the [NUIMTLingual] class
 */
class NUIMTLingual {
  static NUIMTLingual? _instance;
  static const _MODULE = "NUIMTLingual";
  String? _currentSelectedLang; // Current selected language

  late Map<String, NUIMTLanguage> _languageMapping;

  static NUIMTLingual? get(){
    final instance = _instance;
    if(instance == null){
      throwNUI(_MODULE, "Instance have not been built with NUIBusEventBuilder");
    }
    return instance;
  }

  /// Changing the current selected language
  Future changeLang(String lang) async{
    if(!_languageMapping.containsKey(lang)) {
      throwNUI(_MODULE, "Language $lang is not found");
      return;
    }

    _currentSelectedLang = lang;
    await NUISharedPref.get().save("$_MODULE SELECTED_LANG", lang);

    // Sends out a bus event to notify on the change in selected language
    NUIBusEvent.get(module: _MODULE).send(NUIBusEventData<String>(data: lang, sender: _MODULE, code: "Language Change"));
  }

  /// Building the [NUIMTLingual] instance from the builder class
  NUIMTLingual.builder(NUIMTLingualBuilder builder){
    _currentSelectedLang = builder._initialLang;
    _languageMapping = Map();
    for(var l in builder._languages ?? []){
      _languageMapping[l.code] = l;
      _currentSelectedLang = _currentSelectedLang ?? l.code;
    }
    _instance = this;
  }

  /// Getting the current selected language
  Future<String> _getSelectedLanguage() async{
    final sharedPref = await NUISharedPrefBuilder().build();
    return await sharedPref.getString("$_MODULE SELECTED_LANG");
  }

  /// Initialize the [NUIMTLingual] instance
  Future _init() async{
    NUIBusEventBuilder().module(_MODULE).build();
    final curLang = await _getSelectedLanguage();
    if(!isNullOrEmpty(curLang)){
      _currentSelectedLang = curLang;
    }

    // Looping through the list of languages to load the language dictionary
    for(var l in _languageMapping.values){
      if(!isNullOrEmpty(l.assetFilename)){
        // Asset file name provided
        await NUIGlobalConfigBuilder().fileName(l.assetFilename!).module("$_MODULE ${l.code}").build();
      }
      else if(l.dictionary != null){
        // Direct language dictionary
        final config = await NUIGlobalConfigBuilder().module("$_MODULE ${l.code}").build();
        config.loadFromMap(l.dictionary!);
      }
      else if(!isNullOrEmpty(l.externalAssetFilenames)){
        // Last check to see if there are other assets provided
        await NUIGlobalConfigBuilder().fileNames(l.externalAssetFilenames!).module("$_MODULE ${l.code}").build();
      }

      continue;
    }
  }

  /// Open method to get the selected language
  String? get selectedLang{
    return _currentSelectedLang;
  }

  /// Open method to get the list of configured language codes
  List<String> get availableLangs{
    return _languageMapping.keys.toList();
  }

  /// Getting the language dictionary based on the label code provided
  String? lang(String code){
    if(!_languageMapping.containsKey(_currentSelectedLang)) return null;
    return NUIGlobalConfig.get(module: "$_MODULE $_currentSelectedLang")?.getString(code);
  }
}


/**
 * Builder class for creating the [NUIMTLingual] instance
 */
class NUIMTLingualBuilder{
  List<NUIMTLanguage>? _languages; // List of languages that are supported
  String? _initialLang; // Initial language to be rendered

  NUIMTLingualBuilder languages(List<NUIMTLanguage> languages){
    this._languages = languages;
    return this;
  }

  NUIMTLingualBuilder initialLanguage(String initialLanguage){
    this._initialLang = initialLanguage;
    return this;
  }

  Future<NUIMTLingual> build() async{
    final mt = NUIMTLingual.builder(this);
    await mt._init();
    return mt;
  }
}


typedef NUIMTLingualWidgetBuilder = Widget Function(BuildContext context, NUIMTLingual? mtlang, String? lang);
class NUIMTLingualContainer extends StatefulWidget {
  const NUIMTLingualContainer({required this.builder, Key? key}) : super(key: key);
  final NUIMTLingualWidgetBuilder builder;

  @override
  _NUIMTLingualContainerState createState() => _NUIMTLingualContainerState();
}

class _NUIMTLingualContainerState extends State<NUIMTLingualContainer> implements NUIBusEventListener<String> {
  String? _selectedLang;

  @override
  void initState(){
    super.initState();
    _selectedLang = NUIMTLingual.get()?._currentSelectedLang;
    NUIBusEvent.get(module: NUIMTLingual._MODULE).listen(this);
  }

  @override
  Widget build(BuildContext context) {
    return widget.builder(context, NUIMTLingual.get(), _selectedLang);
  }

  @override
  void onBusEventReceived(NUIBusEventData<String?> payload) {
    logNUI("NUIMTLingualContainer", "Received bus event for ${payload.code} with value ${payload.data}");
    if(match(payload.code, "Language Change")){
      setState(() {
        _selectedLang = payload.data;
      });
    }
  }

  @override
  void dispose(){
    super.dispose();
    NUIBusEvent.get(module: NUIMTLingual._MODULE).dispose(this);
  }
}

